var query = new TSqlQueryV2(T_SQLCNT_POOL.getSqlDatabase());
try {
    var uiLoader = this.uiLoader();
    var errLst = uiLoader.validateAll("COMMIT", true, "ERROR");

    if (errLst.length > 0) {
        var errStrLst = [];
        _.forEach(errLst, function (err) {
            errStrLst.push(_.toString(err["text"]));
        }, this);

        this.alertError(this.ttr("Save Failed"), errStrLst.join("\n"));
        return;
    }
    var allData = uiLoader.getAllValues(true);
    var dataMap = {};
    _.forEach(allData, function (v, k) {
        if (_.startsWith(k, "table(slave).")) {
            _.set(dataMap, k.replace("table(slave).", ""), v);
        }
    })
    if (_.isEmpty(_.toString(dataMap["id"]))) {
        // 插入数据
        delete dataMap["id"];
    } else {
        // 更新数据
    }
    var plan_detail = allData["plan_detail"];
    var planDetailIds = _.reduce(plan_detail, function (prev, curr) {
        if (curr.CRUD != 'create' && !_.isEmpty(_.toString(curr.id))) {
            prev.push(curr.id);
        }
        return prev;
    }, [])
    var planDetailGroup = _.groupBy(plan_detail, "CRUD");
    query.begin();
    var id = query.replaceRow({
        table: 'tpm_machine_maintance_plan_v2',
        field: _.keys(dataMap),
        data: dataMap,
        unique_field: ['id'],
        auto_increment_field: 'id'
    });
    
    if (query.lastError().isValid()) {
        throw query.lastError().text();
    }

    if (!_.isEmpty(planDetailIds)) {
        var deleter = new TSqlDeleterV2;
        deleter.setTable("tpm_machine_maintance_plan_detail");
        deleter.setWhere(_.format("id not in ('{0}')", planDetailIds.join("','")));
        deleter.addWhere("plan_id", id);
        query.deleteRow(deleter);
        if (query.lastError().isValid()) {
            throw query.lastError().text();
        }
    }
    _.forEach(planDetailGroup, function (v, k) {
        var dataLst = _.reduce(v, function (prev, curr) {
            curr["plan_id"] = id;
            prev.push(curr);
            return prev;
        }, []);
        if (/create/i.test(k)) {
            query.batchInsert("tpm_machine_maintance_plan_detail", ["plan_id", "item", "method", "spec_value", "max_value", "min_value", "auto_judge_policy"], dataLst);
            if (query.lastError().isValid()) {
                throw query.lastError().text();
            }
        } else if (/update/i.test(k)) {
            var sql = "update tpm_machine_maintance_plan_detail set item = :item, method = :method, spec_value = :spec_value, \
            max_value = :max_value, min_value= :min_value, auto_judge_policy = :auto_judge_policy where id = :id";
            query.batchSql(sql, dataLst);
            if (query.lastError().isValid()) {
                throw query.lastError().text();
            }
        }
    })
    query.commit();
    this.setIsDetailModified(false);
    this.refreshSlave();
} catch (e) {
    query.rollback();
    print(e)
}
/*---ACTION---
ICON: "save"
LABEL: "Save"
LABEL_ZHCN: "保存"
LABEL_ZHTW: "保存"
ACCEL: ""
TOOLTIP: "Save"
TOOLTIP_ZHCN: "保存"
TOOLTIP_ZHTW: "保存"
CHECKED: ""
GROUP: ""
STYLE: "size=small button_style=icon"
LANG: "JavaScript"
STATUS: "Release"
VERSION: "1"
STATEHOOK: "return this.isDetailModified() ? 'enable' : 'disable';"
---ACTION---*/